"""
Basic functionality to test for overlapping 2D shapes.
"""

from math import sqrt

__all__ = (
    'circles_overlap',
    'point_in_polygon',
    'polygons_overlap',
    'rects_overlap'
    )


def circles_overlap(pos1, radius1, pos2, radius2):
    """Return True if two circles overlap"""
    span = radius1 + radius2
    distance = abs(sqrt(
        pow(abs(pos1[0] - pos2[0]), 2) +
        pow(abs(pos1[1] - pos2[1]), 2)
        ))
    return distance <= span

def point_in_rect(point, pos, size):
    """Return True if a point is within the rectangle"""
    return not (point[0] < pos[0] or point[0] > pos[0] + size[0] or point[1] < pos[1] or point[1] > pos[1] + size[1])
    
def rects_overlap(pos1, size1, pos2, size2):
    """Return True if two rectangles overlap. """ 
    return not(pos1[0] + size1[0] < pos2[0] or pos1[0] > pos2[0] + size2[0] or pos1[1] + size1[1] < pos2[1] or pos1[1] > pos2[1] + size2[1])
    
def point_in_polygon(p, points):
    """Return True if a point is within the polygon"""
    
    inside = False
    p1 = points[0]
    point_count = len(points)
    for i in xrange(0, point_count + 1):
        p2 = points[i % point_count]
        if p[1] > min(p1[1], p2[1]) and p[1] <= max(p1[1], p2[1]) and p[0] <= max(p1[0], p2[0]):
            x_intersect = -1e+10
            if p1[1] != p2[1]:
                x_intersect = (p[1] - p1[1]) * (p2[0] - p1[0]) / (p2[1] - p1[1]) + p1[0]

            if p1[0] == p2[0] or p[0] <= x_intersect:
                inside = not inside

        p1 = p2

    return inside
    
def polygons_overlap(points1, points2):
    """Return True if two polygons overlap."""
    for point in points1:
        if point_in_polygon(point, points2):
            return True

    for point in points2:
        if point_in_polygon(point, points1):
            return True