"""
Classes used when the framework throws an exception to something, well at 
least if it's expecting it ;)
"""

__all__ = (
    'ResourceIDNotUnique',
    'UnrecognisedResourceType',
    'UnsupportedDisplayResolution'
    )


class HittException:
    """
    A base class for exceptions thrown by the hitt framework.
    """


class ResourceIDNotUnique(HittException):
    """
    Raised when an attempt is made to define a resource with an ID that is
    already in use.
    """


class ResourceTypeDefined(HittException):
    """
    Raised when an attempt is made to add a resource type that already exists.
    """
        

class UnrecognisedResourceType(HittException):
    """
    Raised when an attempt is made to define a resource of an unknown type.
    """

    
class UnsupportedDisplayResolution(HittException):
    """
    Raised when an attempt is made to set the display to an unsupported 
    resolution.
    """